@@ -19,15 +19,14 @@ module Graphics.UI.GLUT.Callbacks.Registration (
 
 --------------------------------------------------------------------------------
 
-import Control.Monad ( when )
-import Data.IORef ( IORef, newIORef, readIORef, writeIORef, modifyIORef )
+import Control.Monad
+import Data.IORef
 import qualified Data.Map as Map ( empty, lookup, insert, delete )
 import Data.Map ( Map )
-import Foreign.C.Types ( CInt, CUInt )
-import Foreign.Ptr ( FunPtr, nullFunPtr, freeHaskellFunPtr )
-import System.IO.Unsafe ( unsafePerformIO )
-import Graphics.Rendering.OpenGL.GL.StateVar ( HasGetter(get) )
-import Graphics.UI.GLUT.Window ( Window, currentWindow )
+import Data.StateVar
+import Foreign.Ptr
+import Graphics.UI.GLUT.Raw
+import Graphics.UI.GLUT.Window
 
 --------------------------------------------------------------------------------
 -- No timer callback here, because they are one-shot and "self destroy"
@@ -121,25 +120,15 @@ setCleanupList = writeIORef theCleanupList
 -- callback which frees all function pointers on the cleanup list.
 
 {-# NOINLINE theScavenger #-}
-theScavenger :: IORef (FunPtr TimerCallback)
-theScavenger = unsafePerformIO (newIORef =<< makeTimerCallback (\_ -> do
+theScavenger :: IORef (FunPtr TimerFunc)
+theScavenger = unsafePerformIO (newIORef =<< makeTimerFunc (\_ -> do
    cleanupList <- getCleanupList
    mapM_ freeHaskellFunPtr cleanupList
    setCleanupList []))
 
-getScavenger :: IO (FunPtr TimerCallback)
+getScavenger :: IO (FunPtr TimerFunc)
 getScavenger = readIORef theScavenger
 
--- More or less copied from Global.hs to avoid mutual dependencies
-
-type TimerCallback = CInt -> IO ()
-
-foreign import ccall "wrapper" makeTimerCallback ::
-   TimerCallback -> IO (FunPtr TimerCallback)
-
-foreign import CALLCONV unsafe "glutTimerFunc" glutTimerFunc ::
-   CUInt -> FunPtr TimerCallback -> CInt -> IO ()
-
 --------------------------------------------------------------------------------
 -- Here is the really cunning stuff: If an element is added to the cleanup list
 -- when it is empty, register an immediate callback at GLUT to free the list as
