@@ -24,6 +24,15 @@ module Graphics.UI.GLUT.Callbacks.Window (
    -- * Window close callback
    CloseCallback, closeCallback,
 
+   -- * Keyboard callback
+   KeyboardCallback, keyboardCallback, keyboardUpCallback,
+
+   -- * Special callback
+   SpecialCallback, specialCallback, specialUpCallback,
+
+   -- * Mouse callback
+   MouseCallback, mouseCallback,
+
    -- * Keyboard and mouse input callback
    Key(..), SpecialKey(..), MouseButton(..), KeyState(..), Modifiers(..),
    KeyboardMouseCallback, keyboardMouseCallback,
@@ -51,34 +60,16 @@ module Graphics.UI.GLUT.Callbacks.Window (
    JoystickCallback, joystickCallback
 ) where
 
-import Data.Bits ( Bits((.&.)) )
-import Data.Char ( chr )
-import Data.Maybe ( fromJust )
-import Foreign.C.Types ( CInt, CUInt, CUChar )
-import Graphics.Rendering.OpenGL.GL.CoordTrans ( Position(..), Size(..) )
-import Graphics.Rendering.OpenGL.GL.StateVar (
-   SettableStateVar, makeSettableStateVar )
-import Graphics.UI.GLUT.Callbacks.Registration ( CallbackType(..), setCallback )
-import Graphics.UI.GLUT.Constants (
-   glut_NOT_VISIBLE, glut_VISIBLE,
-   glut_HIDDEN, glut_FULLY_RETAINED, glut_PARTIALLY_RETAINED, glut_FULLY_COVERED,
-   glut_KEY_F1, glut_KEY_F2, glut_KEY_F3, glut_KEY_F4, glut_KEY_F5, glut_KEY_F6,
-   glut_KEY_F7, glut_KEY_F8, glut_KEY_F9, glut_KEY_F10, glut_KEY_F11,
-   glut_KEY_F12, glut_KEY_LEFT, glut_KEY_UP, glut_KEY_RIGHT, glut_KEY_DOWN,
-   glut_KEY_PAGE_UP, glut_KEY_PAGE_DOWN, glut_KEY_HOME, glut_KEY_END,
-   glut_KEY_INSERT, glut_KEY_NUM_LOCK, glut_KEY_BEGIN, glut_KEY_DELETE,
-   glut_DOWN, glut_UP,
-   glut_ACTIVE_SHIFT, glut_ACTIVE_CTRL, glut_ACTIVE_ALT,
-   glut_LEFT, glut_ENTERED,
-   glut_JOYSTICK_BUTTON_A, glut_JOYSTICK_BUTTON_B,
-   glut_JOYSTICK_BUTTON_C, glut_JOYSTICK_BUTTON_D )
-import Graphics.UI.GLUT.State ( PollRate )
-import Graphics.UI.GLUT.Types ( MouseButton(..), unmarshalMouseButton )
-import Graphics.UI.GLUT.Extensions
-
---------------------------------------------------------------------------------
-
-#include "HsGLUTExt.h"
+import Data.Bits hiding ( shift )
+import Data.Char
+import Data.Maybe
+import Data.StateVar
+import Foreign.C.Types
+import Graphics.Rendering.OpenGL ( Position(..), Size(..) )
+import Graphics.UI.GLUT.Callbacks.Registration
+import Graphics.UI.GLUT.Raw
+import Graphics.UI.GLUT.State
+import Graphics.UI.GLUT.Types
 
 --------------------------------------------------------------------------------
 
@@ -125,13 +116,7 @@ type DisplayCallback = IO ()
 
 displayCallback :: SettableStateVar DisplayCallback
 displayCallback = makeSettableStateVar $
-   setCallback DisplayCB glutDisplayFunc makeDisplayCallback . Just
-
-foreign import ccall "wrapper" makeDisplayCallback ::
-   DisplayCallback -> IO (FunPtr DisplayCallback)
-
-foreign import CALLCONV unsafe "glutDisplayFunc" glutDisplayFunc ::
-   FunPtr DisplayCallback -> IO ()
+   setCallback DisplayCB glutDisplayFunc makeDisplayFunc . Just
 
 --------------------------------------------------------------------------------
 
@@ -164,10 +149,7 @@ foreign import CALLCONV unsafe "glutDisplayFunc" glutDisplayFunc ::
 
 overlayDisplayCallback :: SettableStateVar (Maybe DisplayCallback)
 overlayDisplayCallback = makeSettableStateVar $
-   setCallback OverlayDisplayCB glutOverlayDisplayFunc makeDisplayCallback
-
-foreign import CALLCONV unsafe "glutOverlayDisplayFunc" glutOverlayDisplayFunc
-   :: FunPtr DisplayCallback -> IO ()
+   setCallback OverlayDisplayCB glutOverlayDisplayFunc makeOverlayDisplayFunc
 
 --------------------------------------------------------------------------------
 
@@ -175,8 +157,6 @@ foreign import CALLCONV unsafe "glutOverlayDisplayFunc" glutOverlayDisplayFunc
 
 type ReshapeCallback = Size -> IO ()
 
-type ReshapeCallback' = CInt -> CInt -> IO ()
-
 -- | Controls the reshape callback for the /current window./ The reshape callback
 -- is triggered when a window is reshaped. A reshape callback is also triggered
 -- immediately before a window\'s first display callback after a window is
@@ -205,15 +185,9 @@ type ReshapeCallback' = CInt -> CInt -> IO ()
 
 reshapeCallback :: SettableStateVar (Maybe ReshapeCallback)
 reshapeCallback = makeSettableStateVar $
-   setCallback ReshapeCB glutReshapeFunc (makeReshapeCallback . unmarshal)
+   setCallback ReshapeCB glutReshapeFunc (makeReshapeFunc . unmarshal)
    where unmarshal cb w h = cb (Size (fromIntegral w) (fromIntegral h))
 
-foreign import ccall "wrapper" makeReshapeCallback ::
-   ReshapeCallback' -> IO (FunPtr ReshapeCallback')
-
-foreign import CALLCONV unsafe "glutReshapeFunc" glutReshapeFunc ::
-   FunPtr ReshapeCallback' -> IO ()
-
 --------------------------------------------------------------------------------
 
 -- | The visibility state of the /current window/
@@ -240,8 +214,6 @@ unmarshalVisibility x
 
 type VisibilityCallback = Visibility -> IO ()
 
-type VisibilityCallback' = CInt -> IO ()
-
 -- | Controls the visibility callback for the /current window./ The visibility
 -- callback for a window is called when the visibility of a window changes.
 --
@@ -256,15 +228,9 @@ type VisibilityCallback' = CInt -> IO ()
 visibilityCallback :: SettableStateVar (Maybe VisibilityCallback)
 visibilityCallback = makeSettableStateVar $
    setCallback VisibilityCB glutVisibilityFunc
-               (makeVisibilityCallback . unmarshal)
+               (makeVisibilityFunc . unmarshal)
    where unmarshal cb  = cb . unmarshalVisibility
 
-foreign import ccall "wrapper" makeVisibilityCallback ::
-   VisibilityCallback' -> IO (FunPtr VisibilityCallback')
-
-foreign import CALLCONV unsafe "glutVisibilityFunc" glutVisibilityFunc ::
-   FunPtr VisibilityCallback' -> IO ()
-
 --------------------------------------------------------------------------------
 
 -- | The window state of the /current window/
@@ -290,9 +256,7 @@ unmarshalWindowState x
 
 type WindowStateCallback = WindowState -> IO ()
 
-type WindowStateCallback_ = CInt -> IO ()
-
--- | (/freeglut only/) Controls the window state callback for the
+-- | Controls the window state callback for the
 -- /current window./ The window state callback for a window is called when the
 -- window state of a window changes.
 --
@@ -307,58 +271,47 @@ type WindowStateCallback_ = CInt -> IO ()
 
 windowStateCallback :: SettableStateVar (Maybe WindowStateCallback)
 windowStateCallback = makeSettableStateVar $
-   setCallback WindowStatusCB glutWindowStateFunc
-               (makeWindowStateCallback . unmarshal)
+   setCallback WindowStatusCB glutWindowStatusFunc
+               (makeWindowStatusFunc . unmarshal)
    where unmarshal cb  = cb . unmarshalWindowState
 
-foreign import ccall "wrapper" makeWindowStateCallback ::
-   WindowStateCallback_ -> IO (FunPtr WindowStateCallback_)
-
-EXTENSION_ENTRY(unsafe,"freeglut",glutWindowStateFunc,FunPtr WindowStateCallback_ -> IO ())
-
 --------------------------------------------------------------------------------
 
 type CloseCallback = IO ()
 
 closeCallback :: SettableStateVar (Maybe CloseCallback)
 closeCallback = makeSettableStateVar $
-   setCallback CloseCB glutCloseFunc makeCloseCallback
-
-foreign import ccall "wrapper"
-   makeCloseCallback :: CloseCallback -> IO (FunPtr CloseCallback)
-
-EXTENSION_ENTRY(unsafe,"freeglut",glutCloseFunc,FunPtr CloseCallback -> IO ())
+   setCallback CloseCB glutCloseFunc makeCloseFunc
 
 --------------------------------------------------------------------------------
 
+-- | A keyboard callback
 type KeyboardCallback = Char -> Position -> IO ()
 
-type KeyboardCallback' = CUChar -> CInt -> CInt -> IO ()
-
 setKeyboardCallback :: Maybe KeyboardCallback -> IO ()
 setKeyboardCallback =
-   setCallback KeyboardCB glutKeyboardFunc (makeKeyboardCallback . unmarshal)
+   setCallback KeyboardCB glutKeyboardFunc (makeKeyboardFunc . unmarshal)
    where unmarshal cb c x y = cb (chr (fromIntegral c))
                                  (Position (fromIntegral x) (fromIntegral y))
 
-foreign import ccall "wrapper" makeKeyboardCallback ::
-   KeyboardCallback' -> IO (FunPtr KeyboardCallback')
-
-foreign import CALLCONV unsafe "glutKeyboardFunc" glutKeyboardFunc ::
-   FunPtr KeyboardCallback' -> IO ()
+-- | Controls the keyboard callback for the /current window/. This is
+-- activated only when a key is pressed.
+keyboardCallback :: SettableStateVar (Maybe KeyboardCallback)
+keyboardCallback = makeSettableStateVar setKeyboardCallback
 
 --------------------------------------------------------------------------------
 
 setKeyboardUpCallback :: Maybe KeyboardCallback -> IO ()
 setKeyboardUpCallback =
    setCallback KeyboardUpCB glutKeyboardUpFunc
-               (makeKeyboardCallback . unmarshal)
+               (makeKeyboardUpFunc . unmarshal)
    where unmarshal cb c x y = cb (chr (fromIntegral c))
                                  (Position (fromIntegral x) (fromIntegral y))
 
-foreign import CALLCONV unsafe "glutKeyboardUpFunc" glutKeyboardUpFunc ::
-   FunPtr KeyboardCallback' -> IO ()
-
+-- | Controls the keyboard callback for the /current window/. This is
+-- activated only when a key is released.
+keyboardUpCallback :: SettableStateVar (Maybe KeyboardCallback)
+keyboardUpCallback = makeSettableStateVar setKeyboardUpCallback
 --------------------------------------------------------------------------------
 
 -- | Special keys
@@ -388,6 +341,7 @@ data SpecialKey
    | KeyNumLock
    | KeyBegin
    | KeyDelete
+   | KeyUnknown Int
    deriving ( Eq, Ord, Show )
 
 unmarshalSpecialKey :: CInt -> SpecialKey
@@ -416,37 +370,35 @@ unmarshalSpecialKey x
    | x == glut_KEY_NUM_LOCK = KeyNumLock
    | x == glut_KEY_BEGIN = KeyBegin
    | x == glut_KEY_DELETE = KeyDelete
-   | otherwise = error ("unmarshalSpecialKey: illegal value " ++ show x)
+   | otherwise = KeyUnknown (fromIntegral x)
 
 --------------------------------------------------------------------------------
 
+-- | A special key callback
 type SpecialCallback = SpecialKey -> Position -> IO ()
 
-type SpecialCallback' = CInt -> CInt -> CInt -> IO ()
-
 setSpecialCallback :: Maybe SpecialCallback -> IO ()
 setSpecialCallback =
-   setCallback SpecialCB glutSpecialFunc (makeSpecialCallback . unmarshal)
+   setCallback SpecialCB glutSpecialFunc (makeSpecialFunc . unmarshal)
    where unmarshal cb k x y = cb (unmarshalSpecialKey k)
                                  (Position (fromIntegral x) (fromIntegral y))
 
-foreign import ccall "wrapper" makeSpecialCallback ::
-   SpecialCallback' -> IO (FunPtr SpecialCallback')
-
-foreign import CALLCONV unsafe "glutSpecialFunc" glutSpecialFunc ::
-   FunPtr SpecialCallback' -> IO ()
-
+-- | Controls the special key callback for the /current window/. This is
+-- activated only when a special key is pressed.
+specialCallback :: SettableStateVar (Maybe SpecialCallback)
+specialCallback = makeSettableStateVar setSpecialCallback
 --------------------------------------------------------------------------------
 
 setSpecialUpCallback :: Maybe SpecialCallback -> IO ()
 setSpecialUpCallback =
-   setCallback SpecialUpCB glutSpecialUpFunc (makeSpecialCallback . unmarshal)
+   setCallback SpecialUpCB glutSpecialUpFunc (makeSpecialUpFunc . unmarshal)
    where unmarshal cb k x y = cb (unmarshalSpecialKey k)
                                  (Position (fromIntegral x) (fromIntegral y))
 
-foreign import CALLCONV unsafe "glutSpecialUpFunc" glutSpecialUpFunc ::
-   FunPtr SpecialCallback' -> IO ()
-
+-- | Controls the special key callback for the /current window/. This is
+-- activated only when a special key is released.
+specialUpCallback :: SettableStateVar (Maybe SpecialCallback)
+specialUpCallback = makeSettableStateVar setSpecialUpCallback
 --------------------------------------------------------------------------------
 
 -- | The current state of a key or button
@@ -464,23 +416,19 @@ unmarshalKeyState x
 
 --------------------------------------------------------------------------------
 
+-- | A mouse callback
 type MouseCallback = MouseButton -> KeyState -> Position -> IO ()
 
-type MouseCallback' = CInt -> CInt -> CInt -> CInt -> IO ()
-
 setMouseCallback :: Maybe MouseCallback -> IO ()
 setMouseCallback =
-   setCallback MouseCB glutMouseFunc (makeMouseCallback . unmarshal)
+   setCallback MouseCB glutMouseFunc (makeMouseFunc . unmarshal)
    where unmarshal cb b s x y = cb (unmarshalMouseButton b)
                                    (unmarshalKeyState s)
                                    (Position (fromIntegral x) (fromIntegral y))
 
-foreign import ccall "wrapper" makeMouseCallback ::
-   MouseCallback' -> IO (FunPtr MouseCallback')
-
-foreign import CALLCONV unsafe "glutMouseFunc" glutMouseFunc ::
-   FunPtr MouseCallback' -> IO ()
-
+-- | Controls the mouse callback for the /current window/.
+mouseCallback :: SettableStateVar (Maybe MouseCallback)
+mouseCallback = makeSettableStateVar setMouseCallback
 --------------------------------------------------------------------------------
 
 -- | The state of the keyboard modifiers
@@ -498,8 +446,6 @@ unmarshalModifiers m = Modifiers {
 getModifiers :: IO Modifiers
 getModifiers = fmap unmarshalModifiers glutGetModifiers
 
-foreign import CALLCONV unsafe "glutGetModifiers" glutGetModifiers :: IO CInt
-
 --------------------------------------------------------------------------------
 
 -- | A generalized view of keys
@@ -520,6 +466,10 @@ type KeyboardMouseCallback =
 -- mouse button changes. The callback parameters indicate the new state of the
 -- key\/button, the state of the keyboard modifiers, and the mouse location in
 -- window relative coordinates.
+--
+-- Note that this is a convenience function that should not ordinarily be used
+-- in conjunction with `keyboardCallback`, `keyboardUpCallback`,
+-- `specialCallback`, `specialUpCallback`, or `mouseCallback`.
 
 keyboardMouseCallback :: SettableStateVar (Maybe KeyboardMouseCallback)
 keyboardMouseCallback = makeSettableStateVar setKeyboardMouseCallback
@@ -551,8 +501,6 @@ type WheelDirection = Int
 
 type MouseWheelCallback = WheelNumber -> WheelDirection -> Position -> IO ()
 
-type MouseWheelCallback_ = CInt -> CInt -> CInt -> CInt -> IO ()
-
 -- | (/freeglut only/) Controls the mouse wheel callback for the
 -- /current window./ The mouse wheel callback for a window is called when a
 -- mouse wheel is used and the wheel number is greater than or equal to
@@ -560,23 +508,16 @@ type MouseWheelCallback_ = CInt -> CInt -> CInt -> CInt -> IO ()
 
 mouseWheelCallback :: SettableStateVar (Maybe MouseWheelCallback)
 mouseWheelCallback = makeSettableStateVar $
-   setCallback MouseWheelCB glutMouseWheelFunc (makeMouseWheelCallback . unmarshal)
+   setCallback MouseWheelCB glutMouseWheelFunc (makeMouseWheelFunc . unmarshal)
    where unmarshal cb n d x y = cb (fromIntegral n) (fromIntegral d)
                                    (Position (fromIntegral x) (fromIntegral y))
 
-foreign import ccall "wrapper" makeMouseWheelCallback ::
-   MouseWheelCallback_ -> IO (FunPtr MouseWheelCallback_)
-
-EXTENSION_ENTRY(unsafe,"freeglut",glutMouseWheelFunc,FunPtr MouseWheelCallback_ -> IO ())
-
 --------------------------------------------------------------------------------
 
 -- | A motion callback
 
 type MotionCallback = Position -> IO ()
 
-type MotionCallback' = CInt -> CInt -> IO ()
-
 -- | Controls the motion callback for the /current window./ The motion callback
 -- for a window is called when the mouse moves within the window while one or
 -- more mouse buttons are pressed. The callback parameter indicates the mouse
@@ -584,15 +525,9 @@ type MotionCallback' = CInt -> CInt -> IO ()
 
 motionCallback :: SettableStateVar (Maybe MotionCallback)
 motionCallback = makeSettableStateVar $
-   setCallback MotionCB glutMotionFunc (makeMotionCallback . unmarshal)
+   setCallback MotionCB glutMotionFunc (makeMotionFunc . unmarshal)
    where unmarshal cb x y = cb (Position (fromIntegral x) (fromIntegral y))
 
-foreign import ccall "wrapper" makeMotionCallback ::
-   MotionCallback' -> IO (FunPtr MotionCallback')
-
-foreign import CALLCONV unsafe "glutMotionFunc" glutMotionFunc ::
-   FunPtr MotionCallback' -> IO ()
-
 --------------------------------------------------------------------------------
 
 -- | Controls the passive motion callback for the /current window./ The passive
@@ -603,12 +538,9 @@ foreign import CALLCONV unsafe "glutMotionFunc" glutMotionFunc ::
 passiveMotionCallback :: SettableStateVar (Maybe MotionCallback)
 passiveMotionCallback = makeSettableStateVar $
    setCallback PassiveMotionCB glutPassiveMotionFunc
-               (makeMotionCallback . unmarshal)
+               (makePassiveMotionFunc . unmarshal)
    where unmarshal cb x y = cb (Position (fromIntegral x) (fromIntegral y))
 
-foreign import CALLCONV unsafe "glutPassiveMotionFunc" glutPassiveMotionFunc ::
-   FunPtr MotionCallback' -> IO ()
-
 --------------------------------------------------------------------------------
 
 -- | The relation between the mouse pointer and the /current window/ has
@@ -631,8 +563,6 @@ unmarshalCrossing x
 
 type CrossingCallback = Crossing -> IO ()
 
-type CrossingCallback' = CInt -> IO ()
-
 -- | Controls the mouse enter\/leave callback for the /current window./ Note
 -- that some window systems may not generate accurate enter\/leave callbacks.
 --
@@ -641,15 +571,9 @@ type CrossingCallback' = CInt -> IO ()
 
 crossingCallback :: SettableStateVar (Maybe CrossingCallback)
 crossingCallback = makeSettableStateVar $
-   setCallback CrossingCB glutEntryFunc (makeCrossingCallback . unmarshal)
+   setCallback CrossingCB glutEntryFunc (makeEntryFunc . unmarshal)
    where unmarshal cb = cb . unmarshalCrossing
 
-foreign import ccall "wrapper" makeCrossingCallback ::
-   CrossingCallback' -> IO (FunPtr CrossingCallback')
-
-foreign import CALLCONV unsafe "glutEntryFunc" glutEntryFunc ::
-   FunPtr CrossingCallback' -> IO ()
-
 --------------------------------------------------------------------------------
 
 -- | Translation of the Spaceball along one axis, normalized to be in the range
@@ -709,16 +633,10 @@ type SpaceballMotionCallback =
 setSpaceballMotionCallback :: Maybe SpaceballMotionCallback -> IO ()
 setSpaceballMotionCallback =
    setCallback SpaceballMotionCB glutSpaceballMotionFunc
-               (makeSpaceballMotionCallback . unmarshal)
+               (makeSpaceballMotionFunc . unmarshal)
    where unmarshal cb x y z =
             cb (fromIntegral x) (fromIntegral y) (fromIntegral z)
 
-foreign import ccall "wrapper" makeSpaceballMotionCallback ::
-   SpaceballMotionCallback -> IO (FunPtr SpaceballMotionCallback)
-
-foreign import CALLCONV unsafe "glutSpaceballMotionFunc" glutSpaceballMotionFunc
-   :: FunPtr SpaceballMotionCallback -> IO ()
-
 --------------------------------------------------------------------------------
 
 type SpaceballRotationCallback =
@@ -727,34 +645,20 @@ type SpaceballRotationCallback =
 setSpaceballRotationCallback :: Maybe SpaceballRotationCallback -> IO ()
 setSpaceballRotationCallback =
    setCallback SpaceballRotateCB glutSpaceballRotateFunc
-               (makeSpaceballRotationCallback . unmarshal)
+               (makeSpaceballRotateFunc . unmarshal)
    where unmarshal cb x y z =
             cb (fromIntegral x) (fromIntegral y) (fromIntegral z)
 
-foreign import ccall "wrapper" makeSpaceballRotationCallback ::
-   SpaceballRotationCallback -> IO (FunPtr SpaceballRotationCallback)
-
-foreign import CALLCONV unsafe "glutSpaceballRotateFunc" glutSpaceballRotateFunc
-   :: FunPtr SpaceballRotationCallback -> IO ()
-
 --------------------------------------------------------------------------------
 
 type SpaceballButtonCallback = ButtonIndex -> KeyState -> IO ()
 
-type SpaceballButtonCallback' = CInt -> CInt -> IO ()
-
 setSpaceballButtonCallback :: Maybe SpaceballButtonCallback -> IO ()
 setSpaceballButtonCallback =
    setCallback SpaceballButtonCB glutSpaceballButtonFunc
-               (makeSpaceballButtonCallback . unmarshal)
+               (makeSpaceballButtonFunc . unmarshal)
    where unmarshal cb b s = cb (fromIntegral b) (unmarshalKeyState s)
 
-foreign import ccall "wrapper" makeSpaceballButtonCallback ::
-   SpaceballButtonCallback' -> IO (FunPtr SpaceballButtonCallback')
-
-foreign import CALLCONV unsafe "glutSpaceballButtonFunc"
-   glutSpaceballButtonFunc :: FunPtr SpaceballButtonCallback' -> IO ()
-
 --------------------------------------------------------------------------------
 
 -- | The index of a specific dial of a dial and button box.
@@ -798,36 +702,20 @@ setDialAndButtonBoxCallback (Just cb) = do
 
 type ButtonBoxCallback = ButtonIndex -> KeyState -> IO ()
 
-type ButtonBoxCallback' = CInt -> CInt -> IO ()
-
 setButtonBoxCallback :: Maybe ButtonBoxCallback -> IO ()
 setButtonBoxCallback =
    setCallback ButtonBoxCB glutButtonBoxFunc (makeButtonBoxFunc . unmarshal)
    where unmarshal cb b s = cb (fromIntegral b) (unmarshalKeyState s)
 
-foreign import ccall "wrapper" makeButtonBoxFunc ::
-   ButtonBoxCallback' -> IO (FunPtr ButtonBoxCallback')
-
-foreign import CALLCONV unsafe "glutButtonBoxFunc" glutButtonBoxFunc ::
-   FunPtr ButtonBoxCallback' -> IO ()
-
 --------------------------------------------------------------------------------
 
 type DialsCallback = DialIndex -> Int -> IO ()
 
-type DialsCallback' = CInt -> CInt -> IO ()
-
 setDialsCallback :: Maybe DialsCallback -> IO ()
 setDialsCallback =
     setCallback DialsCB glutDialsFunc (makeDialsFunc . unmarshal)
     where unmarshal cb d x = cb (fromIntegral d) (fromIntegral x)
 
-foreign import ccall "wrapper" makeDialsFunc ::
-   DialsCallback -> IO (FunPtr DialsCallback')
-
-foreign import CALLCONV unsafe "glutDialsFunc" glutDialsFunc ::
-   FunPtr DialsCallback' -> IO ()
-
 --------------------------------------------------------------------------------
 
 -- | Absolute tablet position, with coordinates normalized to be in the range of
@@ -872,8 +760,6 @@ setTabletCallback (Just cb) = do
 
 type TabletMotionCallback = TabletPosition -> IO ()
 
-type TabletMotionCallback' = CInt -> CInt -> IO ()
-
 setTabletMotionCallback :: Maybe TabletMotionCallback -> IO ()
 setTabletMotionCallback =
     setCallback TabletMotionCB glutTabletMotionFunc
@@ -881,18 +767,10 @@ setTabletMotionCallback =
     where unmarshal cb x y =
              cb (TabletPosition (fromIntegral x) (fromIntegral y))
 
-foreign import ccall "wrapper" makeTabletMotionFunc ::
-   TabletMotionCallback' -> IO (FunPtr TabletMotionCallback')
-
-foreign import CALLCONV unsafe "glutTabletMotionFunc" glutTabletMotionFunc ::
-   FunPtr TabletMotionCallback' -> IO ()
-
 --------------------------------------------------------------------------------
 
 type TabletButtonCallback = ButtonIndex -> KeyState -> TabletPosition -> IO ()
 
-type TabletButtonCallback' = CInt -> CInt -> CInt -> CInt -> IO ()
-
 setTabletButtonCallback :: Maybe TabletButtonCallback -> IO ()
 setTabletButtonCallback =
     setCallback TabletButtonCB glutTabletButtonFunc
@@ -901,12 +779,6 @@ setTabletButtonCallback =
              cb (fromIntegral b) (unmarshalKeyState s)
                 (TabletPosition (fromIntegral x) (fromIntegral y))
 
-foreign import ccall "wrapper" makeTabletButtonFunc ::
-   TabletButtonCallback' -> IO (FunPtr TabletButtonCallback')
-
-foreign import CALLCONV unsafe "glutTabletButtonFunc" glutTabletButtonFunc ::
-   FunPtr TabletButtonCallback' -> IO ()
-
 --------------------------------------------------------------------------------
 
 -- | The state of the joystick buttons
@@ -945,8 +817,6 @@ data JoystickPosition = JoystickPosition Int Int Int
 
 type JoystickCallback = JoystickButtons -> JoystickPosition -> IO ()
 
-type JoystickCallback' = CUInt -> CInt -> CInt -> CInt -> IO ()
-
 -- | Controls the joystick callback for the /current window./ The joystick
 -- callback is called either due to polling of the joystick at the uniform timer
 -- interval specified (if > 0) or in response to an explicit call of
@@ -966,9 +836,3 @@ joystickCallback =
                                     (JoystickPosition (fromIntegral x)
                                                       (fromIntegral y)
                                                       (fromIntegral z))
-
-foreign import ccall "wrapper" makeJoystickFunc ::
-   JoystickCallback' -> IO (FunPtr JoystickCallback')
-
-foreign import CALLCONV unsafe "glutJoystickFunc" glutJoystickFunc ::
-   FunPtr JoystickCallback' -> CInt -> IO ()
