@@ -17,10 +17,10 @@ module Graphics.UI.GLUT.Debugging (
    reportErrors
 ) where
 
-import System.Environment ( getProgName )
-import System.IO ( hPutStrLn, stderr )
-import Graphics.Rendering.OpenGL.GL.StateVar ( HasGetter(get) )
-import Graphics.Rendering.OpenGL.GLU.Errors ( Error(..), errors )
+import Data.StateVar
+import Graphics.Rendering.OpenGL ( Error(..), errors )
+import System.Environment
+import System.IO
 
 --------------------------------------------------------------------------------
 
