@@ -45,47 +45,25 @@ module Graphics.UI.GLUT.Initialization (
    DirectRendering(..), directRendering,
 
    -- * OpenGL 3.x context support
-   initialContextVersion, ContextFlag(..), initialContextFlags
+   initialContextVersion, ContextFlag(..), initialContextFlags,
+   ContextProfile(..), initialContextProfile
 ) where
 
-import Control.Monad ( when )
-import Data.Bits ( Bits((.|.),(.&.)), complement )
-import Data.List ( genericLength, intersperse, mapAccumR )
-import Foreign.C.String ( CString, withCString, peekCString )
-import Foreign.C.Types ( CInt, CUInt )
-import Foreign.Marshal.Array ( withArray0, peekArray )
-import Foreign.Marshal.Utils ( with, withMany )
-import Foreign.Ptr ( Ptr, nullPtr, nullFunPtr )
-import Foreign.Storable ( Storable(..) )
-import System.Environment ( getProgName, getArgs )
-import Graphics.Rendering.OpenGL.GL.CoordTrans ( Position(..), Size(..) )
-import Graphics.Rendering.OpenGL.GL.StateVar (
-   GettableStateVar, makeGettableStateVar,
-   SettableStateVar, makeSettableStateVar,
-   StateVar, makeStateVar, HasGetter(..), HasSetter(..) )
-import Graphics.UI.GLUT.Constants (
-   glut_INIT_WINDOW_X, glut_INIT_WINDOW_Y,
-   glut_INIT_WINDOW_WIDTH, glut_INIT_WINDOW_HEIGHT,
-   glut_RGBA, glut_RGB, glut_INDEX, glut_LUMINANCE,
-   glut_ALPHA, glut_ACCUM, glut_DEPTH, glut_STENCIL,
-   glut_AUX1, glut_AUX2, glut_AUX3, glut_AUX4,
-   glut_SINGLE, glut_DOUBLE, glut_MULTISAMPLE, glut_STEREO,
-   glut_CAPTIONLESS, glut_BORDERLESS,
-   glut_INIT_DISPLAY_MODE,
-   glut_DISPLAY_MODE_POSSIBLE,
-   glut_RENDERING_CONTEXT, glut_CREATE_NEW_CONTEXT, glut_USE_CURRENT_CONTEXT,
-   glut_DIRECT_RENDERING,
-   glut_FORCE_INDIRECT_CONTEXT, glut_ALLOW_DIRECT_CONTEXT,
-   glut_TRY_DIRECT_CONTEXT, glut_FORCE_DIRECT_CONTEXT,
-   glut_INIT_MAJOR_VERSION, glut_INIT_MINOR_VERSION,
-   glut_DEBUG, glut_FORWARD_COMPATIBLE, glut_INIT_FLAGS )
-import Graphics.UI.GLUT.Extensions
-import Graphics.UI.GLUT.QueryUtils ( simpleGet, glutSetOption )
-import Graphics.UI.GLUT.Types ( Relation(..), relationToString )
-
---------------------------------------------------------------------------------
-
-#include "HsGLUTExt.h"
+import Control.Monad
+import Data.Bits
+import Data.List
+import Data.StateVar
+import Foreign.C.String
+import Foreign.C.Types
+import Foreign.Marshal.Array
+import Foreign.Marshal.Utils
+import Foreign.Ptr
+import Foreign.Storable
+import Graphics.Rendering.OpenGL ( Position(..), Size(..) )
+import Graphics.UI.GLUT.QueryUtils
+import Graphics.UI.GLUT.Raw
+import Graphics.UI.GLUT.Types
+import System.Environment
 
 --------------------------------------------------------------------------------
 
@@ -145,9 +123,6 @@ initialize prog args =
    newArgv <- mapM peekCString newArgvPtrs
    return $ tail newArgv
 
-foreign import CALLCONV unsafe "glutInit" glutInit ::
-   Ptr CInt -> Ptr CString -> IO ()
-
 -- | Convenience action: Initialize GLUT, returning the program name and any
 -- non-GLUT command line arguments.
 
@@ -166,8 +141,6 @@ getArgsAndInitialize = do
 exit :: IO ()
 exit = glutExit
 
-EXTENSION_ENTRY(unsafe,"freeglut",glutExit,IO ())
-
 --------------------------------------------------------------------------------
 
 -- | Controls the /initial window position/.  Windows created by
@@ -196,9 +169,6 @@ setInitialWindowPosition :: Position -> IO ()
 setInitialWindowPosition (Position x y) =
     glutInitWindowPosition (fromIntegral x) (fromIntegral y)
 
-foreign import CALLCONV unsafe "glutInitWindowPosition" glutInitWindowPosition
-   :: CInt -> CInt -> IO ()
-
 --------------------------------------------------------------------------------
 
 -- | Controls the /initial window size/.  Windows created by
@@ -228,9 +198,6 @@ setInitialWindowSize :: Size -> IO ()
 setInitialWindowSize (Size w h) =
    glutInitWindowSize (fromIntegral w) (fromIntegral h)
 
-foreign import CALLCONV unsafe "glutInitWindowSize" glutInitWindowSize ::
-   CInt -> CInt -> IO ()
-
 --------------------------------------------------------------------------------
 
 -- | A single aspect of a window which is to be created, used in conjunction
@@ -287,6 +254,8 @@ data DisplayMode
      -- ^ Select a window without a caption (/freeglut only/).
    | Borderless
      -- ^ Select a window without any borders (/freeglut only/).
+   | SRGBMode
+     -- ^ Select an sRGB mode window (/freeglut only/).
    deriving ( Eq, Ord, Show )
 
 marshalDisplayMode :: DisplayMode -> CUInt
@@ -312,6 +281,7 @@ marshalDisplayMode m = case m of
    Stereoscopic -> glut_STEREO
    Captionless -> glut_CAPTIONLESS
    Borderless -> glut_BORDERLESS
+   SRGBMode -> glut_SRGB
 
 --------------------------------------------------------------------------------
 
@@ -348,7 +318,7 @@ i2dmsWithoutRGBA bitfield =
                 WithAccumBuffer, WithDepthBuffer, WithStencilBuffer,
                 WithAuxBuffers 1, WithAuxBuffers 2, WithAuxBuffers 3,
                 WithAuxBuffers 4, SingleBuffered, DoubleBuffered, Multisampling,
-                Stereoscopic, Captionless, Borderless ]
+                Stereoscopic, Captionless, Borderless, SRGBMode ]
        , (bitfield .&. marshalDisplayMode c) /= 0 ]
 
 setInitialDisplayMode :: [DisplayMode] -> IO ()
@@ -361,12 +331,9 @@ handleMultisampling :: [Int] -> DisplayMode -> ([Int], DisplayMode)
 handleMultisampling spps (WithSamplesPerPixel spp) = (spp : spps, Multisampling)
 handleMultisampling spps mode                      = (spps, mode)
 
-toBitfield :: (Bits b) => (a -> b) -> [a] -> b
+toBitfield :: (Num b, Bits b) => (a -> b) -> [a] -> b
 toBitfield marshal = foldl (.|.) 0 . map marshal
 
-foreign import CALLCONV unsafe "glutInitDisplayMode" glutInitDisplayMode ::
-   CUInt -> IO ()
-
 -- | Contains 'True' if the /current display mode/ is supported, 'False'
 -- otherwise.
 
@@ -397,7 +364,7 @@ setSamplesPerPixel spp = do
 
 multisamplingSupported :: IO Bool
 multisamplingSupported = isKnown "glutGetModeValues"
-   where isKnown = fmap (/= nullFunPtr) . getProcAddressInternal
+   where isKnown = fmap (/= nullFunPtr) . getAPIEntryInternal
 
 --------------------------------------------------------------------------------
 
@@ -609,9 +576,6 @@ initialDisplayCapabilities =
           caps)
          glutInitDisplayString
 
-foreign import CALLCONV unsafe "glutInitDisplayString" glutInitDisplayString ::
-  CString -> IO ()
-
 -----------------------------------------------------------------------------
 
 -- | How rendering context for new windows are created.
@@ -724,8 +688,6 @@ setContextVersion :: (Int, Int) -> IO ()
 setContextVersion (major, minor) =
    glutInitContextVersion (fromIntegral major) (fromIntegral minor)
 
-EXTENSION_ENTRY(unsafe,"freeglut",glutInitContextVersion,CInt -> CInt -> IO ())
-
 -----------------------------------------------------------------------------
 
 -- | A flag affecting the rendering context to create, used in conjunction
@@ -769,4 +731,40 @@ i2cfs bitfield =
 setContextFlags :: [ContextFlag] -> IO ()
 setContextFlags = glutInitContextFlags . toBitfield marshalContextFlag
 
-EXTENSION_ENTRY(unsafe,"freeglut",glutInitContextFlags,CInt -> IO ())
+
+-----------------------------------------------------------------------------
+
+-- | An OpenGL API profile, affecting the rendering context to create, used
+-- in conjunction with 'initialContextProfile'.
+
+data ContextProfile
+   = -- | The OpenGL /core/ profile, which all OpenGL 3.2 implementations
+     -- are required to support.
+      CoreProfile
+   | -- | The OpenGL /compatibility/ profile, which is optional for OpenGL
+     -- 3.2 implementations.
+     CompatibilityProfile
+   deriving ( Eq, Ord, Show )
+
+marshalContextProfile :: ContextProfile -> CInt
+marshalContextProfile x = case x of
+   CoreProfile -> glut_CORE_PROFILE
+   CompatibilityProfile -> glut_COMPATIBILITY_PROFILE
+
+-----------------------------------------------------------------------------
+
+-- | (/freeglut only/) Controls the set of profiles for the rendering context.
+
+initialContextProfile :: StateVar [ContextProfile]
+initialContextProfile = makeStateVar getContextProfiles setContextProfiles
+
+getContextProfiles :: IO [ContextProfile]
+getContextProfiles = simpleGet i2cps glut_INIT_PROFILE
+
+i2cps :: CInt -> [ContextProfile]
+i2cps bitfield =
+   [ c | c <- [ CoreProfile, CompatibilityProfile ]
+       , (fromIntegral bitfield .&. marshalContextProfile c) /= 0 ]
+
+setContextProfiles :: [ContextProfile] -> IO ()
+setContextProfiles = glutInitContextProfile . toBitfield marshalContextProfile
