@@ -52,32 +52,13 @@ module Graphics.UI.GLUT.Window (
    Cursor(..), cursor, pointerPosition
 ) where
 
-import Foreign.C.String ( CString, withCString )
-import Foreign.C.Types ( CInt )
-import Graphics.Rendering.OpenGL.GL.CoordTrans ( Position(..), Size(..) )
-import Graphics.Rendering.OpenGL.GL.StateVar (
-   GettableStateVar, makeGettableStateVar,
-   SettableStateVar, makeSettableStateVar,
-   StateVar, makeStateVar )
-import Graphics.UI.GLUT.Constants (
-   glut_WINDOW_PARENT, glut_WINDOW_NUM_CHILDREN,
-   glut_WINDOW_X, glut_WINDOW_Y, glut_WINDOW_WIDTH, glut_WINDOW_HEIGHT,
-   glut_CURSOR_RIGHT_ARROW, glut_CURSOR_LEFT_ARROW, glut_CURSOR_INFO,
-   glut_CURSOR_DESTROY, glut_CURSOR_HELP, glut_CURSOR_CYCLE, glut_CURSOR_SPRAY,
-   glut_CURSOR_WAIT, glut_CURSOR_TEXT, glut_CURSOR_CROSSHAIR,
-   glut_CURSOR_UP_DOWN, glut_CURSOR_LEFT_RIGHT, glut_CURSOR_TOP_SIDE,
-   glut_CURSOR_BOTTOM_SIDE, glut_CURSOR_LEFT_SIDE, glut_CURSOR_RIGHT_SIDE,
-   glut_CURSOR_TOP_LEFT_CORNER, glut_CURSOR_TOP_RIGHT_CORNER,
-   glut_CURSOR_BOTTOM_RIGHT_CORNER, glut_CURSOR_BOTTOM_LEFT_CORNER,
-   glut_CURSOR_INHERIT, glut_CURSOR_NONE, glut_CURSOR_FULL_CROSSHAIR,
-   glut_WINDOW_CURSOR )
-import Graphics.UI.GLUT.Extensions
-import Graphics.UI.GLUT.QueryUtils ( simpleGet )
-import Graphics.UI.GLUT.Types ( Window, makeWindow )
-
---------------------------------------------------------------------------------
-
-#include "HsGLUTExt.h"
+import Data.StateVar
+import Foreign.C.String
+import Foreign.C.Types
+import Graphics.Rendering.OpenGL ( Position(..), Size(..) )
+import Graphics.UI.GLUT.QueryUtils
+import Graphics.UI.GLUT.Raw
+import Graphics.UI.GLUT.Types
 
 --------------------------------------------------------------------------------
 
@@ -108,10 +89,7 @@ import Graphics.UI.GLUT.Types ( Window, makeWindow )
 createWindow
    :: String    -- ^ The window name
    -> IO Window -- ^ The identifier for the newly created window
-createWindow name = withCString name glutCreateWindow
-
-foreign import CALLCONV unsafe "glutCreateWindow" glutCreateWindow ::
-      CString -> IO Window
+createWindow name = fmap Window $ withCString name glutCreateWindow
 
 --------------------------------------------------------------------------------
 
@@ -125,13 +103,11 @@ createSubWindow
                 --   origin.
    -> Size      -- ^ Window size in pixels
    -> IO Window -- ^ The identifier for the newly created subwindow
-createSubWindow win (Position x y) (Size w h) =
-   glutCreateSubWindow win
-                       (fromIntegral x) (fromIntegral y)
-                       (fromIntegral w) (fromIntegral h)
-
-foreign import CALLCONV unsafe "glutCreateSubWindow" glutCreateSubWindow ::
-      Window -> CInt -> CInt -> CInt -> CInt -> IO Window
+createSubWindow (Window win) (Position x y) (Size w h) =
+   fmap Window $
+      glutCreateSubWindow win
+                          (fromIntegral x) (fromIntegral y)
+                          (fromIntegral w) (fromIntegral h)
 
 --------------------------------------------------------------------------------
 
@@ -141,7 +117,7 @@ foreign import CALLCONV unsafe "glutCreateSubWindow" glutCreateSubWindow ::
 parentWindow :: GettableStateVar (Maybe Window)
 parentWindow =
    makeGettableStateVar $
-      getWindow (simpleGet makeWindow glut_WINDOW_PARENT)
+      getWindow (simpleGet Window glut_WINDOW_PARENT)
 
 --------------------------------------------------------------------------------
 
@@ -162,8 +138,8 @@ numSubWindows =
 -- /current window/, the /current window/ becomes invalid ('currentWindow' will
 -- contain 'Nothing').
 
-foreign import CALLCONV unsafe "glutDestroyWindow" destroyWindow ::
-   Window -> IO ()
+destroyWindow :: Window -> IO ()
+destroyWindow (Window win) = glutDestroyWindow win
 
 --------------------------------------------------------------------------------
 
@@ -174,16 +150,14 @@ foreign import CALLCONV unsafe "glutDestroyWindow" destroyWindow ::
 
 currentWindow :: StateVar (Maybe Window)
 currentWindow =
-   makeStateVar (getWindow glutGetWindow) (maybe (return ()) glutSetWindow)
+   makeStateVar
+      (getWindow (fmap Window glutGetWindow))
+      (maybe (return ()) (\(Window win) -> glutSetWindow win))
 
 getWindow :: IO Window -> IO (Maybe Window)
 getWindow act = do
    win <- act
-   return $ if win == makeWindow 0 then Nothing else Just win
-
-foreign import CALLCONV unsafe "glutSetWindow" glutSetWindow :: Window -> IO ()
-
-foreign import CALLCONV unsafe "glutGetWindow" glutGetWindow :: IO Window
+   return $ if win == Window 0 then Nothing else Just win
 
 --------------------------------------------------------------------------------
 
@@ -203,9 +177,7 @@ foreign import CALLCONV unsafe "glutGetWindow" glutGetWindow :: IO Window
 -- Also, see 'Graphics.UI.GLUT.Overlay.postOverlayRedisplay'.
 
 postRedisplay :: Maybe Window -> IO ()
-postRedisplay = maybe glutPostRedisplay glutPostWindowRedisplay
-
-foreign import CALLCONV unsafe "glutPostRedisplay" glutPostRedisplay :: IO ()
+postRedisplay = maybe glutPostRedisplay (\(Window win) -> glutPostWindowRedisplay win)
 
 -- | Mark the normal plane of the given window as needing to be redisplayed,
 -- otherwise the same as 'postRedisplay'.
@@ -215,9 +187,6 @@ foreign import CALLCONV unsafe "glutPostRedisplay" glutPostRedisplay :: IO ()
 -- particularly useful when multiple windows need redisplays posted at the same
 -- time.
 
-foreign import CALLCONV unsafe "glutPostWindowRedisplay"
-   glutPostWindowRedisplay :: Window -> IO ()
-
 --------------------------------------------------------------------------------
 
 -- | Perform a buffer swap on the /layer in use/ for the /current window/.
@@ -234,7 +203,8 @@ foreign import CALLCONV unsafe "glutPostWindowRedisplay"
 --
 -- If the /layer in use/ is not double buffered, 'swapBuffers' has no effect.
 
-foreign import CALLCONV unsafe "glutSwapBuffers" swapBuffers :: IO ()
+swapBuffers :: IO ()
+swapBuffers = glutSwapBuffers
 
 --------------------------------------------------------------------------------
 
@@ -266,9 +236,6 @@ setWindowPosition :: Position -> IO ()
 setWindowPosition (Position x y) =
    glutPositionWindow (fromIntegral x) (fromIntegral y)
 
-foreign import CALLCONV unsafe "glutPositionWindow" glutPositionWindow ::
-   CInt -> CInt -> IO ()
-
 getWindowPosition :: IO Position
 getWindowPosition = do
    x <- simpleGet fromIntegral glut_WINDOW_X
@@ -294,9 +261,6 @@ setWindowSize :: Size -> IO ()
 setWindowSize (Size w h) =
    glutReshapeWindow (fromIntegral w) (fromIntegral h)
 
-foreign import CALLCONV unsafe "glutReshapeWindow" glutReshapeWindow ::
-   CInt -> CInt -> IO ()
-
 getWindowSize :: IO Size
 getWindowSize = do
    w <- simpleGet fromIntegral glut_WINDOW_WIDTH
@@ -320,7 +284,8 @@ getWindowSize = do
 -- absolutely no decorations. Non-Motif window managers may not respond to
 -- @_MOTIF_WM_HINTS@.
 
-foreign import CALLCONV unsafe "glutFullScreen" fullScreen :: IO ()
+fullScreen :: IO ()
+fullScreen = glutFullScreen
 
 --------------------------------------------------------------------------------
 
@@ -329,8 +294,6 @@ foreign import CALLCONV unsafe "glutFullScreen" fullScreen :: IO ()
 fullScreenToggle :: IO ()
 fullScreenToggle = glutFullScreenToggle
 
-EXTENSION_ENTRY(unsafe,"freeglut",glutFullScreenToggle,IO ())
-
 --------------------------------------------------------------------------------
 
 -- $ManipulatingTheStackingOrder
@@ -344,12 +307,14 @@ EXTENSION_ENTRY(unsafe,"freeglut",glutFullScreenToggle,IO ())
 -- | Change the stacking order of the /current window/ relative to its siblings
 -- (lowering it).
 
-foreign import CALLCONV unsafe "glutPushWindow" pushWindow :: IO ()
+pushWindow :: IO ()
+pushWindow = glutPushWindow
 
 -- | Change the stacking order of the /current window/ relative to its siblings,
 -- bringing the /current window/ closer to the top.
 
-foreign import CALLCONV unsafe "glutPopWindow" popWindow :: IO ()
+popWindow :: IO ()
+popWindow = glutPopWindow
 
 --------------------------------------------------------------------------------
 
@@ -376,12 +341,6 @@ windowStatus = makeSettableStateVar setStatus
          setStatus Hidden    = glutHideWindow
          setStatus Iconified = glutIconifyWindow
 
-foreign import CALLCONV unsafe "glutShowWindow" glutShowWindow :: IO ()
-
-foreign import CALLCONV unsafe "glutHideWindow" glutHideWindow :: IO ()
-
-foreign import CALLCONV unsafe "glutIconifyWindow" glutIconifyWindow :: IO ()
-
 --------------------------------------------------------------------------------
 
 -- $ChangingTheWindowIconTitle
@@ -401,9 +360,6 @@ windowTitle =
    makeSettableStateVar $ \name ->
       withCString name glutSetWindowTitle
 
-foreign import CALLCONV unsafe "glutSetWindowTitle" glutSetWindowTitle ::
-      CString -> IO ()
-
 -- | Controls the icon title of the /current top-level window/.
 
 iconTitle :: SettableStateVar String
@@ -411,9 +367,6 @@ iconTitle =
    makeSettableStateVar $ \name ->
       withCString name glutSetIconTitle
 
-foreign import CALLCONV unsafe "glutSetIconTitle" glutSetIconTitle ::
-      CString -> IO ()
-
 --------------------------------------------------------------------------------
 
 -- | The different cursor images GLUT supports.
@@ -515,8 +468,6 @@ cursor = makeStateVar getCursor setCursor
 setCursor :: Cursor -> IO ()
 setCursor = glutSetCursor . marshalCursor
 
-foreign import CALLCONV unsafe "glutSetCursor" glutSetCursor :: CInt -> IO ()
-
 getCursor :: IO Cursor
 getCursor = simpleGet unmarshalCursor glut_WINDOW_CURSOR
 
@@ -539,6 +490,3 @@ pointerPosition :: SettableStateVar Position
 pointerPosition =
    makeSettableStateVar $ \(Position x y) ->
       glutWarpPointer (fromIntegral x) (fromIntegral y)
-
-foreign import CALLCONV unsafe "glutWarpPointer" glutWarpPointer ::
-   CInt -> CInt -> IO ()
